' See Eric W. Weisstein's article "Newton's Method."
' From MathWorld--A Wolfram Web Resource. 
' http://mathworld.wolfram.com/NewtonsMethod.html 

Imports System.Math

Public Class Form1
    Inherits System.Windows.Forms.Form

#Region " Windows Form Designer generated code "

    Public Sub New()
        MyBase.New()

        'This call is required by the Windows Form Designer.
        InitializeComponent()

        'Add any initialization after the InitializeComponent() call

    End Sub

    'Form overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    'Required by the Windows Form Designer
    Private components As System.ComponentModel.IContainer

    'NOTE: The following procedure is required by the Windows Form Designer
    'It can be modified using the Windows Form Designer.  
    'Do not modify it using the code editor.
    Friend WithEvents picCanvas As System.Windows.Forms.PictureBox
    Friend WithEvents MainMenu1 As System.Windows.Forms.MainMenu
    Friend WithEvents MenuItem1 As System.Windows.Forms.MenuItem
    Friend WithEvents mnuFullScale As System.Windows.Forms.MenuItem
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        Me.picCanvas = New System.Windows.Forms.PictureBox
        Me.MainMenu1 = New System.Windows.Forms.MainMenu
        Me.MenuItem1 = New System.Windows.Forms.MenuItem
        Me.mnuFullScale = New System.Windows.Forms.MenuItem
        Me.SuspendLayout()
        '
        'picCanvas
        '
        Me.picCanvas.Dock = System.Windows.Forms.DockStyle.Fill
        Me.picCanvas.Location = New System.Drawing.Point(0, 0)
        Me.picCanvas.Name = "picCanvas"
        Me.picCanvas.Size = New System.Drawing.Size(292, 273)
        Me.picCanvas.TabIndex = 0
        Me.picCanvas.TabStop = False
        '
        'MainMenu1
        '
        Me.MainMenu1.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.MenuItem1})
        '
        'MenuItem1
        '
        Me.MenuItem1.Index = 0
        Me.MenuItem1.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.mnuFullScale})
        Me.MenuItem1.Text = "&Scale"
        '
        'mnuFullScale
        '
        Me.mnuFullScale.Index = 0
        Me.mnuFullScale.Text = "&Full Scale"
        '
        'Form1
        '
        Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
        Me.ClientSize = New System.Drawing.Size(292, 273)
        Me.Controls.Add(Me.picCanvas)
        Me.Menu = Me.MainMenu1
        Me.Name = "Form1"
        Me.Text = "Newton's Method"
        Me.ResumeLayout(False)

    End Sub

#End Region

#Region "Variables"
    ' True after the initial Show.
    Private m_FormVisible As Boolean = False

    ' Variables for rubberband drawing.
    Private m_DrawingBox As Boolean
    Private m_X1, m_Y1, m_X2, m_Y2 As Integer
    Private m_ZoomingBitmap As Bitmap
    Private m_ZoomingGraphics As Graphics

    ' Drawing variables.
    Private m_Bitmap As Bitmap
    Private m_Graphics As Graphics

    ' World coordinates.
    Private Const MIN_X As Double = -1.5
    Private Const MAX_X As Double = 1.5
    Private Const MIN_Y As Double = -1.5
    Private Const MAX_Y As Double = 1.5
    Private m_Wxmin As Double = MIN_X
    Private m_Wxmax As Double = MAX_X
    Private m_Wymin As Double = MIN_Y
    Private m_Wymax As Double = MAX_Y

    ' Colors for up to 8 roots.
    ' Black = exceeded MAX_ITER iterations.
    ' White = unknown root.
    Private m_Colors() As Color = { _
        Color.Black, _
        Color.White, _
        Color.Red, _
        Color.Green, _
        Color.Blue, _
        Color.Yellow, _
        Color.Orange, _
        Color.Cyan, _
        Color.Magenta, _
        Color.Lime _
    }

#End Region ' Variables

    Private Class Complex
        Public Re As Double
        Public Im As Double
        Public Sub New(ByVal real_part As Double, ByVal imaginary_part As Double)
            Re = real_part
            Im = imaginary_part
        End Sub
        Public Function Clone() As Complex
            Return New Complex(Re, Im)
        End Function
        Public Function MagnitudeSquared() As Double
            Return Re * Re + Im * Im
        End Function
        Public Function Magnitude() As Double
            Return Sqrt(MagnitudeSquared)
        End Function
        Public Function Times(ByVal x As Complex) As Complex
            Return New Complex(Re * x.Re - Im * x.Im, Re * x.Im + Im * x.Re)
        End Function
        Public Function Times(ByVal new_re As Double, ByVal new_im As Double) As Complex
            Return New Complex(Re * new_re - Im * new_im, Re * new_im + Im * new_re)
        End Function
        Public Function Plus(ByVal x As Complex) As Complex
            Return New Complex(Re + x.Re, Im + x.Im)
        End Function
        Public Function Plus(ByVal new_re As Double, ByVal new_im As Double) As Complex
            Return New Complex(Re + new_re, Im + new_im)
        End Function
        Public Function DividedBy(ByVal denominator As Complex) As Complex
            Dim conjugate As New Complex(denominator.Re, -denominator.Im)
            denominator = denominator.Times(conjugate)
            Dim numerator As Complex = Me.Times(conjugate)
            numerator.Re /= denominator.Re
            numerator.Im /= denominator.Re
            Return numerator
        End Function
        Public Function IsCloseTo(ByVal x As Complex) As Boolean
            Return Abs(Re - x.Re) + Abs(Im - x.Im) < 0.001
        End Function
        Public Overrides Function ToString() As String
            If Im < 0 Then
                Return Re.ToString() & " - " & Abs(Im).ToString() & "i"
            Else
                Return Re.ToString() & " + " & Im.ToString() & "i"
            End If
        End Function
    End Class

#Region "Function"
    Private Const NUM_ROOTS As Integer = 7
    Private m_Roots(NUM_ROOTS) As Complex

    ' The function.
    ' F(x) = x^NUM_ROOTS - 1.
    Private Function F(ByVal x As Complex) As Complex
        Dim result As Complex = x.Clone()   ' x.

        ' x^NUM_ROOTS.
        For i As Integer = 1 To NUM_ROOTS - 1
            result = result.Times(x)
        Next i

        ' x^NUM_ROOTS - 1.
        Return result.Plus(-1, 0)
    End Function

    ' The function's derivative.
    ' dFdx(x) = NUM_ROOTS * x^(NUM_ROOTS - 1).
    Private Function dFdx(ByVal x As Complex) As Complex
        Dim result As Complex = x.Clone()   ' x.

        ' x^(NUM_ROOTS - 1).
        For i As Integer = 1 To NUM_ROOTS - 2
            result = result.Times(x)
        Next i

        ' NUM_ROOTS * x^(NUM_ROOTS - 1).
        Return result.Times(NUM_ROOTS, 0)
    End Function
#End Region ' Function

    ' Draw the first image.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Display the form.
        Me.Show()
        m_FormVisible = True

        ' Initialize the roots.
        For i As Integer = 0 To NUM_ROOTS - 1
            m_Roots(i) = New Complex( _
                Cos(2 * i * PI / NUM_ROOTS), _
                Sin(2 * i * PI / NUM_ROOTS))
        Next i

        ' Draw at the current (full) scale.
        DrawFractal()
    End Sub

    ' Redraw with the new form size.
    Private Sub Form1_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Resize
        ' If the form is minimized, do nothing.
        If Me.WindowState = FormWindowState.Minimized Then Exit Sub

        ' See if we just moved and didn't resize.
        If Not (m_Bitmap Is Nothing) Then
            ' We have a Bitmap. 
            ' If it's already the right size, do nothing.
            If m_Bitmap.Width = picCanvas.ClientSize.Width And _
               m_Bitmap.Height = picCanvas.ClientSize.Height _
                Then Exit Sub
        End If

        ' Draw at the current scale.
        DrawFractal()
    End Sub

#Region "Rubberbanding"
    ' Start rubberband drawing.
    Private Sub picCanvas_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseDown
        m_DrawingBox = True
        m_X1 = e.X
        m_Y1 = e.Y
        m_X2 = m_X1
        m_Y2 = m_Y1

        ' Make a copy of the current bitmap.
        m_ZoomingBitmap = New Bitmap(m_Bitmap)
        m_ZoomingGraphics = Graphics.FromImage(m_ZoomingBitmap)
    End Sub

    ' Continue rubberband drawing.
    Private Sub picCanvas_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseMove
        If Not m_DrawingBox Then Exit Sub

        ' Save the new corner.
        m_X2 = e.X
        m_Y2 = e.Y

        ' Draw the new box.
        Dim gr As Graphics = picCanvas.CreateGraphics()
        Dim rect As New Rectangle
        rect.X = Min(m_X1, m_X2)
        rect.Y = Min(m_Y1, m_Y2)
        rect.Width = Abs(m_X2 - m_X1)
        rect.Height = Abs(m_Y2 - m_Y1)
        m_ZoomingGraphics.DrawImage(m_Bitmap, 0, 0)
        m_ZoomingGraphics.DrawRectangle(Pens.White, rect)

        ' Display the result.
        picCanvas.Image = m_ZoomingBitmap
    End Sub

    ' Finish rubberband drawing.
    Private Sub picCanvas_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseUp
        If Not m_DrawingBox Then Exit Sub
        m_DrawingBox = False

        ' Dispose of the zooming Bitmap and Graphics.
        picCanvas.Image = Nothing
        m_ZoomingBitmap.Dispose()
        m_ZoomingGraphics.Dispose()
        m_ZoomingBitmap = Nothing
        m_ZoomingGraphics = Nothing

        ' Save the new corner.
        m_X2 = e.X
        m_Y2 = e.Y

        ' Put the selected coordinates in order.
        Dim x1, x2, y1, y2, factor As Double
        x1 = Min(m_X1, m_X2)
        x2 = Max(m_X1, m_X2)
        y1 = Min(m_Y1, m_Y2)
        y2 = Max(m_Y1, m_Y2)

        ' Convert screen coords into drawing coords.
        factor = (m_Wxmax - m_Wxmin) / picCanvas.ClientSize.Width
        m_Wxmax = m_Wxmin + x2 * factor
        m_Wxmin = m_Wxmin + x1 * factor

        factor = (m_Wymax - m_Wymin) / picCanvas.ClientSize.Height
        m_Wymax = m_Wymin + y2 * factor
        m_Wymin = m_Wymin + y1 * factor

        ' Draw the new fractal.
        DrawFractal()
    End Sub
#End Region ' Rubberbanding

    ' Adjust the aspect ratio of the selected
    ' coordinates so they fit the window properly.
    Private Sub AdjustAspect()
        Dim want_aspect, picCanvas_aspect As Double
        Dim hgt, wid, mid As Double

        want_aspect = (m_Wymax - m_Wymin) / (m_Wxmax - m_Wxmin)
        picCanvas_aspect = picCanvas.ClientSize.Height / picCanvas.ClientSize.Width
        If want_aspect > picCanvas_aspect Then
            ' The selected area is too tall and thin.
            ' Make it wider.
            wid = (m_Wymax - m_Wymin) / picCanvas_aspect
            mid = (m_Wxmin + m_Wxmax) / 2
            m_Wxmin = mid - wid / 2
            m_Wxmax = mid + wid / 2
        Else
            ' The selected area is too short and wide.
            ' Make it taller.
            hgt = (m_Wxmax - m_Wxmin) * picCanvas_aspect
            mid = (m_Wymin + m_Wymax) / 2
            m_Wymin = mid - hgt / 2
            m_Wymax = mid + hgt / 2
        End If
    End Sub

    ' Draw the fractal.
    Private Sub DrawFractal()
        ' Do nothing when the form is initially created.
        If Not m_FormVisible Then Exit Sub

        ' Do nothing if the canvas has zero size.
        If picCanvas.ClientSize.Width < 1 Or _
           picCanvas.ClientSize.Height < 1 Then Exit Sub

        ' Create the Bitmap and Graphic objects.
        If Not (m_Graphics Is Nothing) Then
            picCanvas.Image = Nothing
            m_Graphics.Dispose()
            m_Bitmap.Dispose()
        End If
        m_Bitmap = New Bitmap(picCanvas.ClientSize.Width, picCanvas.ClientSize.Height)
        m_Graphics = Graphics.FromImage(m_Bitmap)

        ' Clear the bitmap.
        m_Graphics.Clear(Me.BackColor)
        picCanvas.Image = m_Bitmap
        picCanvas.Refresh()

        ' Work until the epsilon squared < this.
        Const CUTOFF As Single = 0.0000001

        ' Adjust the coordinate bounds to fit picCanvas.
        AdjustAspect()

        Dim clr As Integer
        Dim x0, x, epsilon As Complex
        Dim dx, dy As Double

        ' dx and dy are the changes in the real 
        ' and imaginary parts across pixels.
        Dim wid As Integer = picCanvas.ClientSize.Width
        Dim hgt As Integer = picCanvas.ClientSize.Height
        dx = (m_Wxmax - m_Wxmin) / (wid - 1)
        dy = (m_Wymax - m_Wymin) / (hgt - 1)

        ' Calculate the values.
        x0 = New Complex(m_Wxmin, 0)
        For i As Integer = 0 To wid - 1
            x0.Im = m_Wymin
            For j As Integer = 0 To hgt - 1
                x = x0
                Const MAX_ITER As Integer = 200
                Dim iter As Integer = 0
                Do
                    iter += 1
                    If iter > MAX_ITER Then Exit Do
                    epsilon = F(x).DividedBy(dFdx(x)).Times(-1, 0)
                    x = x.Plus(epsilon)
                Loop While epsilon.MagnitudeSquared() > CUTOFF

                ' Set which root this is.
                If iter > MAX_ITER Then
                    clr = 0 ' Black.
                Else
                    ' Find the root.
                    clr = 1 ' White.
                    For r As Integer = 0 To m_Roots.GetUpperBound(0)
                        If x.IsCloseTo(m_Roots(r)) Then
                            clr = r + 2 ' +2 to skip black and white.
                            Exit For
                        End If
                    Next r
                End If

                ' Set the pixel's color.
                m_Bitmap.SetPixel(i, j, m_Colors(clr))

                ' Move to the next point.
                x0.Im += dy
            Next j
            x0.Re += dx

            ' Let the user know we're not dead.
            If i Mod 10 = 0 Then picCanvas.Refresh()
        Next i

        ' Update the image.
        picCanvas.Refresh()

        Me.Text = "Newton's Method (" & _
            Format$(m_Wxmin) & ", " & _
            Format$(m_Wymin) & ")-(" & _
            Format$(m_Wxmax) & ", " & _
            Format$(m_Wymax) & ")"
    End Sub

    ' Return to full scale.
    Private Sub mnuFullScale_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFullScale.Click
        m_Wxmin = MIN_X
        m_Wxmax = MAX_X
        m_Wymin = MIN_Y
        m_Wymax = MAX_Y

        DrawFractal()
    End Sub
End Class
